// S88 DI board
// Copyright(C)2016- Desktop Station
// reference : RB_S88, Fujigaya's S88 Control board
// 
//160512 Yaasan    Improve coding rule and add S88 DI features


#define		PORT_DATAIN		5
#define		PORT_DATAOUT	6
#define		PORT_BTNA		A0
#define		PORT_BTNB		A1
#define		PORT_LED1		8
#define		PORT_LED2		9
#define		PORT_LED3		10
#define		PORT_LED4		11
#define		PORT_LED5		A2
#define		PORT_LED6		A3
#define		PORT_LED7		A4
#define		PORT_LED8		A5

#define		BTNA_SW1		0b00000001
#define		BTNA_SW2		0b00000010
#define		BTNA_SW3		0b00000100
#define		BTNA_SW4		0b00001000
#define		BTNA_SW5		0b00010000
#define		BTNA_SW6		0b00000001
#define		BTNA_SW7		0b00000010
#define		BTNA_SW8		0b00000100
#define		BTNA_SW9		0b00001000
#define		BTNA_SW10		0b00010000


//Sensor control
#define		 INTERVAL_L5	100 //L5 interval[ms]
#define		 MAX_S88PORTNUM	8 //port_num to see from DCC Controller

byte gClockCounter = 0;
unsigned long gPreviousL5 = 0;

//Logic IC emulation buffer
unsigned long gTransferData = 0; // same as shift register value
word gSensorData = 0;   // same as flip flop buffer
byte gStatusData[8] = {0,0,0,0,0,0,0,0};


//Button buffer
byte gBtnDataA = 0;
byte gBtnDataB = 0;


/* Functions */
void ControlLEDs();
void ProcessCheck(byte inBufA, byte inBufB, byte inTempA, byte inTempB);
byte GetButtonsFromAD(unsigned short inADData);
void MonitorButtons();

/***********************************************************************
 * 
 *  Functions
 * 
***********************************************************************/

void setup() 
{
	
	Serial.begin(115200);
	while (!Serial);

	Serial.println("--------------------------------------");
	Serial.println("Desktop Station S88 DI board V0.1     ");
	Serial.println("--------------------------------------");
	Serial.println("100 Ready");	
	
	//S88 Control
	attachInterrupt(0, irqCLK, RISING); //pin 2 = clock interrupt
	attachInterrupt(1, irqPS, RISING);  //pin 3 = PS interrupt
	pinMode(2, INPUT_PULLUP);
	pinMode(3, INPUT_PULLUP);
	pinMode(PORT_DATAIN, INPUT_PULLUP); // data in from S88 in chain
	pinMode(PORT_DATAOUT, OUTPUT); // data out to commnad station or to previous S88 chain
	digitalWrite(PORT_DATAOUT, LOW);   

	//Buttons
	pinMode(PORT_BTNA, INPUT);
	pinMode(PORT_BTNB, INPUT);

	//LEDs
	pinMode(PORT_LED1, OUTPUT);
	pinMode(PORT_LED2, OUTPUT);
	pinMode(PORT_LED3, OUTPUT);
	pinMode(PORT_LED4, OUTPUT);
	pinMode(PORT_LED5, OUTPUT);
	pinMode(PORT_LED6, OUTPUT);
	pinMode(PORT_LED7, OUTPUT);
	pinMode(PORT_LED8, OUTPUT);
	
	//Task base
	gPreviousL5 = millis();

	//init sensor_value,transfer_value
	gTransferData = 0;
	gSensorData = 0;

}

/* Main loop */
void loop()
{
	
	//Interval 100ms
	if(millis() - gPreviousL5 > INTERVAL_L5)
	{
		MonitorButtons();
		
		ControlLEDs();
		
		gPreviousL5 = millis();
	}
	
	
}

void irqPS()
{
	//Set transfer value
	gClockCounter = 0;
	
	//Set Shift data
	gTransferData = gSensorData;
	
}

void irqCLK()
{
	//transfer anf emulate shift register.
	
	//Write transfer_value
	digitalWrite(PORT_DATAOUT, bitRead(gTransferData, gClockCounter));
	
	//Delay (e.g. 20ms)
	delayMicroseconds(16); 
	
	//Store PORT_DATAIN value
	bitWrite(gTransferData, gClockCounter, digitalRead(PORT_DATAIN));
	
	//Increment clock
	gClockCounter++;
	
	if( gClockCounter >= MAX_S88PORTNUM)
	{
		gClockCounter = 0;
	}
}


void ControlLEDs()
{
	
	digitalWrite(PORT_LED1, bitRead(gSensorData, 0));
	digitalWrite(PORT_LED2, bitRead(gSensorData, 1));
	digitalWrite(PORT_LED3, bitRead(gSensorData, 2));
	digitalWrite(PORT_LED4, bitRead(gSensorData, 3));
	digitalWrite(PORT_LED5, bitRead(gSensorData, 4));
	digitalWrite(PORT_LED6, bitRead(gSensorData, 5));
	digitalWrite(PORT_LED7, bitRead(gSensorData, 6));
	digitalWrite(PORT_LED8, bitRead(gSensorData, 7));
	
}

void MonitorButtons()
{
	byte aTempBtnA = 0;
	byte aTempBtnB = 0;
	
	unsigned short aButtonA = analogRead(PORT_BTNA);
	unsigned short aButtonB = analogRead(PORT_BTNB);
	
	//Get button status
	aTempBtnA = GetButtonsFromAD(aButtonA);
	aTempBtnB = GetButtonsFromAD(aButtonB);
	
	//Check and process the buttons
	ProcessCheck(gBtnDataA, gBtnDataB, aTempBtnA, aTempBtnB);
	
	//Store to buffer
	gBtnDataA = aTempBtnA;
	gBtnDataB = aTempBtnB;
	
}

void ProcessCheck(byte inBufA, byte inBufB, byte inTempA, byte inTempB)
{
	byte aChanged = 0;
	
	if( (inBufA == 0) && (inTempA > 0))
	{
		
		switch(inTempA)
		{
		case 1:
			//SW1
			gStatusData[0] = (gStatusData[0] == 0) ? 1 : 0;
			break;
		case 2:
			//SW2
			gStatusData[1] = (gStatusData[1] == 0) ? 1 : 0;
			break;
		case 4:
			//SW3
			gStatusData[2] = (gStatusData[2] == 0) ? 1 : 0;
			break;
		case 8:
			//SW4
			gStatusData[3] = (gStatusData[3] == 0) ? 1 : 0;
			break;
		case 16:
			//SW5
			gStatusData[4] = (gStatusData[4] == 0) ? 1 : 0;
			break;
		}
		
		aChanged++;
		
	}
	
	if( (inBufB == 0) && (inTempB > 0))
	{
		
		switch(inTempB)
		{
		case 1:
			//SW6
			gStatusData[5] = (gStatusData[5] == 0) ? 1 : 0;
			break;
		case 2:
			//SW7
			gStatusData[6] = (gStatusData[6] == 0) ? 1 : 0;
			break;
		case 4:
			//SW8
			gStatusData[7] = (gStatusData[7] == 0) ? 1 : 0;
			break;
		case 8:
			//SW9
			gSensorData = 0;
			break;
		case 16:
			//SW10
			gSensorData = 0b11111111;
			break;
		}
		
		aChanged++;
	}
	
	if( aChanged > 0)
	{
		gSensorData = 0;
		
		for( int i = 0; i < 8; i++)
		{
			if( gStatusData[i] > 0)
			{
				gSensorData = gSensorData + (1 << i);
			}
			
		}
		
		
		
		Serial.print("BUTTON: ");
		Serial.println(gSensorData);
	}
	
}

byte GetButtonsFromAD(unsigned short inADData)
{
	/*
	R1	2.2k
	R2	470
	R3	1k
	R4	2.2k
	R5	6.8k
	
	Threshold value:
	SW1: 82
	SW2: 290
	SW3: 525
	SW4: 747
	SW5: 942
	
	
	*/
	
	byte aRet = 0;
	
	if( inADData <= 82)
	{
		aRet = 1 << 0;
	}
	else if( inADData <= 290)
	{
		aRet = 1 << 1;
	}
	else if( inADData <= 525)
	{
		aRet = 1 << 2;
	}
	else if( inADData <= 747)
	{
		aRet = 1 << 3;
	}
	else if( inADData <= 942)
	{
		aRet = 1 << 4;
	}
	else
	{
		//Not pressed
	}
	
	
	return aRet;
}



